import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQuick.Layouts  1.12
import QtQuick.Dialogs 1.3
import QtQml 2.12

import com.st.motorcontrol.qml 1.0
import com.st.motorcontrol.motorcontrolregister 1.0
import "."

//Documentation
//https://doc.qt.io/qt-5/qtqml-index.html
//All QML types : https://doc.qt.io/qt-5/qmltypes.html


//create frame to enclose all controls
Frame {
    id: surface

    /* Motor Control Application parameters */
        property int minimumSpeed: -4000
        property int maximumSpeed: 4000
        property int startupSpeed: 500
        property int nomimalVBus: 24
        property int maximumVBus: 30
        property int minimumVBus: 10
    	property int maximumPower: 10
        property int maximumTemperature: 100
        property string registerFileDescription: "RegListSTMV2.json"
        property string version : "v1.51"

    font.pixelSize: 12

    width: 900
    height: 350

    //Enable polling of register and disable EEPROM function
    Component.onCompleted:
    {
        mcRegBank.enablePolling(true)
        mcMain.enableEEPROM(false)
    }

    Label{
        id: labelversion
        text: surface.version
        y : surface.height - 30
        x:0
    }



    //Create a groupBox for Button
    GroupBox{
        id: groupBoxControl
        title: "CONTROL"
        //Create a Column Layout for Start and Stop Button
        ColumnLayout{
            id: cmdControl


            //Create Start Button
            Button{
                id:startButton
                text: 'START'
                onClicked: mcApplication.startMotor( 1 )
                Layout.fillWidth: true
            }

            //Create Stop Button
            Button{
                id:stopButton
                text: 'STOP'
                onClicked:  mcApplication.stopMotor(1)
                Layout.fillWidth: true
            }

            Label { text: "Target Speed [rpm]" }

            SpinBox {
                id: rampTargetSpeed

                // TBD: change
                property real rampTargetSpeedRegVal: {
                    if (mcRegBank.getRegisterByName( "SPEED_RAMP" ))
                    {
                        if (mcRegBank.getRegisterByName( "SPEED_RAMP" ).values.length > 0)
                            return mcRegBank.getRegisterByName( "SPEED_RAMP" ).values[0]
                    }
                    return 0
                }

                from: surface.minimumSpeed
                to: surface.maximumSpeed

                contentItem: TextInput {
                    text: rampTargetSpeed.value
                    readOnly: ! rampTargetSpeed.editable
                    selectByMouse: true

                    font: rampTargetSpeed.font
                    horizontalAlignment: Qt.AlignRight

                    Keys.onPressed: {
                        if ((event.key === Qt.Key_Enter) || (event.key === Qt.Key_Return)) {
                        	console.log("Execute Speed Ramp fired!!!")
                            mcRegBank.getRegisterByName( "SPEED_RAMP" ).setValue( [ Number(text), 0] ) //speedRampDuration.value  ] )
                        }
                    }
                }

                editable: true
                wheelEnabled: true

                onRampTargetSpeedRegValChanged: {
                    value = rampTargetSpeedRegVal
                }

                Component.onCompleted: value = surface.startupSpeed
            }

            Label { text: "Ramp duration [ms]" }

            SpinBox {
                id: speedRampDuration

                from: 0
                to: 2000000000
                stepSize: 100

                contentItem: TextInput {
                    text: speedRampDuration.value
                    readOnly: ! speedRampDuration.editable
                    selectByMouse: true

                    font: speedRampDuration.font
                    horizontalAlignment: Qt.AlignRight

                    Keys.onPressed: {
                        if ((event.key === Qt.Key_Enter) || (event.key === Qt.Key_Return)) {
                        	console.log("Execute Speed Ramp fired!!!")
                            mcRegBank.getRegisterByName( "SPEED_RAMP" ).setValue( [ rampTargetSpeed.value, Number(text) ] )
                        }
                    }
                }

                editable: true
                wheelEnabled: true
            }

            Button {
                text: "Execute Speed Ramp"

                Layout.columnSpan: 3
                Layout.fillWidth: true

                onClicked: {
                    console.log("Execute Speed Ramp fired!!!")
                    mcRegBank.getRegisterByName( "SPEED_RAMP" ).setValue( [ rampTargetSpeed.value, speedRampDuration.value ] )
                }

            }

            Rectangle {
                Layout.fillWidth: true
                height: 1
                color: surface.palette.button
            }


        }
    }

	GroupBox{
		id:vbusGaugeGroupBox
		
		anchors.left: groupBoxControl.right
        
		McRegGaugeVertical{
			id : vbusGauge
			regName: "BUS_VOLTAGE"

		}
	}


    //Create a groupbox of Register using McRegView component
    GroupBox
    {
        anchors.left: vbusGaugeGroupBox.right
        id: groupBoxreg

        title: "Register"
        ColumnLayout
        {
            McRegView
            {
                regName: "MOTOR_POWER"
                widthHint: "0123456"
            }
            McRegView
            {
                regName: "SPEED_REF"
                widthHint: "0123456"
            }
            McRegView
            {
                regName: "SPEED_MEAS"
                widthHint: "0123456"
            }
        }
    }

    McGroupBoxFWConfig
    {
        anchors.left: groupboxStatus.right
        anchors.top: groupboxStatus.top
        id:groupBoxconfig
    }

//    GroupBox
//    {
//        anchors.left: groupboxStatus.right
//        anchors.top: groupboxStatus.top
//        //anchors.right: groupboxStatus.left
//        id:groupBoxconfig

//        title: "MC Configuration"
//        ColumnLayout
//        {

//            Rectangle
//            {

//                Connections {
//                    target: mcMain
//                    onConnectionStatusChanged: {
//                        console.log( "onConnectionStatusChanged: " + status )

//                        if ( status === 1 /*mcMain.CONNECTED*/ ) {
//                            console.log( "onConnectionStatusChanged: CONNECTED" )
//                            config1.buildText( config1Reg.reg.values )
//                            config2.buildText( config2Reg.reg.values )
//                            config3.buildText( config3Reg.reg.values )

//                        } else if ( status === 3 /*mcMain.TIMEOUT*/ ) {
//                            console.log( "onConnectionStatusChanged: TIMEOUT" )
//                        } else if ( status === 2 /*mcMain.DISCONNECTED*/ ) {
//                            console.log( "onConnectionStatusChanged: DISCONNECTED" )
//                            config1.buildText( ["-","-","-","-","-"] )
//                            config2.buildText( ["-","-","-","-","-"] )
//                            config3.buildText( ["-","-","-","-","-","-"] )
//                        } else {
//                            console.log( "onConnectionStatusChanged: BOUSE " + status )
//                        }
//                    }
//                }
//            }

//            //Label { text: "MC Configuration" }

//            Text {
//                id: config1
//                text: qsTr(config1.buildText( ["-","-","-","-","-"] ))

//                function buildText(values) {
//                    config1.text = "Version: " + (values[3]) + "." + (values[2]) + "." + (values[1]) + "\n"
//                                    +"Motors: " + (values[4])


//                    console.log("values=" + values)
//                }

//                McRegProxy {
//                    id: config1Reg
//                    name: "GLOBAL_CONFIG"

//                    onValuesChanged: {
//                        config1.buildText(values)
//                        console.log( "UPDATED CONFIG: " + values )
//                    }
//                }
//            }

//            Text {
//                id: config2
//                text: qsTr(config2.buildText( ["-","-","-","-","-"] ))

//                function buildText(values) {
//                    config2.text = "Max App Speed: " + (values[0]?values[0]:"?") + " RPM\n"
//                                 + "Max App Current: " + (values[1]?values[1]:"?") + " A\n"
//                                 + "Max App Voltage: " + (values[2]?values[2]:"?") + " V\n"
//                                 + "Min App Voltage: " + (values[3]?values[3]:"?") + " V\n"
//                                 + "Drive Type: " + values[4]
//                }

//                McRegProxy {
//                    id: config2Reg
//                    name: "MOTOR_CONFIG"

//                    onValuesChanged: {
//                        config2.buildText( values )

//                                            //if this register is not correctly filled in FW value can be too high and qml take too much time (or crash)  to adapt gauge

//                        console.log( "UPDATED MOTOR CONFIG: " + values )
//                                            if ( config2Reg.reg.values[0] < 200000 &&  config2Reg.reg.values[0] >0) {
//                                                surface.minimumSpeed = - config2Reg.reg.values[0];
//                                                surface.maximumSpeed = config2Reg.reg.values[0];
//                                            }


//                                            if ( config2Reg.reg.values[2] < 1000 &&  config2Reg.reg.values[2] >0) {
//                                                console.log( "new max Vbus =  " +  config2Reg.reg.values[2])
//                                                //calculate the closest ten of Vbus
//                                                var maxVbus =config2Reg.reg.values[2];
//                                                var num_digit = Math.round(maxVbus).toString().length;
//                                                var unit = Math.round(maxVbus).toString()[0]
//                                                var vbus_10 = (parseInt(unit,16) + 1) * Math.pow(10,num_digit-1)
//                                                surface.maximumVBus = vbus_10;
//                                                gaugeVerticalVbus.tickmarkStepSize = vbus_10 / 5

//                                            }
//                    }
//                }
//            }


//            Text {
//                id: config3
//                text: qsTr(config3.buildText( ["-","-","-","-","-","-"] ))

//                function buildText(values) {
//                    var t = "Primary speed sensor: "

//                    switch ( values[0] ) {
//                    case 0: t += "No sensor"; break
//                    case 1: t += "STO+PLL"; break;
//                    case 2: t += "STO+Cordic"; break;
//                    case 3: t += "Encoder"; break;
//                    case 4: t += "Hall sensor"; break;
//                    case 5: t += "HSO"; break;
//                    case 6: t += "HSO+ZEST"; break;
//                       default: t+= "-";break;
//                    }

//                    t+= "\nAux speed sensor: "

//                    switch ( values[1] ) {
//                    case 0: t += "No sensor"; break
//                    case 1: t += "STO+PLL"; break;
//                    case 2: t += "STO+Cordic"; break;
//                    case 3: t += "Encoder"; break;
//                    case 4: t += "Hall sensor"; break;
//                    case 5: t += "HSO"; break;
//                    case 6: t += "HSO+ZEST"; break;
//                    }

//                    t += "\nCurrent sensing topology: "

//                    switch( values[2] ) {
//                    case 0: t += "3 Shunt"; break;
//                    case 1: t += "1 Shunt"; break;
//                    case 2: t += "1 Shunt, phase shift"; break;
//                    case 3: t += "ICS"; break;
//                    }

//                    t += "\nFOC rate: " + values[3]
//                    t += "\nPWM frequency: " + values[4]
//                    t += "\nMedium frequency: " + values[5]
//                    if ( values[6] & 1 ) t+= "\nFlux Weakening"
//                    if ( values[6] & 2 ) t+= "\nFeed Forward"
//                    if ( values[6] & 4 ) t+= "\nMTPA"
//                    if ( values[6] & 8 ) t+= "\nPFC"
//                    if ( values[6] & 16 ) t+= "\nICL"
//                    if ( values[6] & 32 ) t+= "\nResistive Brake"
//                    if ( values[6] & 64 ) t+= "\nOCP Disable"
//                    if ( values[6] & 128 ) t+= "\nSTGAP"
//                    if ( values[6] & 256 ) t+= "\nPosition Control"
//                    if ( values[6] & 512 ) t+= "\nVBus Sensing"
//                    if ( values[6] & 1024 ) t+= "\nTemperature Sensing"
//                    if ( values[6] & 2048 ) t+= "\nVoltage Sensing"
//                    if ( values[6] & 4096 ) t+= "\nFlash Config"
//                                    if ( values[6] & 8192 ) {
//                                        dac1RowLayout.enabled = true
//                                        t+= "\nDAC Channel 1"
//                                    }
//                                    else{
//                                        dac1RowLayout.enabled = false
//                                    }

//                                    if ( values[6] & 16384 ) {
//                                        dac2RowLayout.enabled = true
//                                        t+= "\nDAC Channel 2"
//                                    }
//                                    else{
//                                        dac2RowLayout.enabled = false
//                                    }

//                    if ( values[7] & 1 ) t+= "\nMCP over STLink"
//                    if ( values[7] & 2 ) t+= "\nMCPA over STLink"
//                    if ( values[7] & 4 ) t+= "\nMCP over UARTA"
//                    if ( values[7] & 8 ) t+= "\nMCPA over UARTA"
//                    if ( values[7] & 16 ) t+= "\nMCP over UARTB"
//                    if ( values[7] & 32 ) t+= "\nMCPA over UARTB"

//                                    mcRegBank.setFWConfig(t);
//                                    console.log( "UPDATED FOC FW CONFIG text= " + t )



//                    config3.text = t
//                }

//                McRegProxy {
//                    id: config3Reg
//                    name: "FOC_FW_CONFIG"

//                    onValuesChanged: {
//                        config3.buildText( values )

//                        console.log( "UPDATED FOC FW CONFIG: " + values )
//                    }
//                }
//            }
//        }

//    }

    //Create status gouptbox using MCStatusBox component
    McStatusBox {
        id : groupboxStatus
        title: "Status"
        anchors.left: groupBoxreg.right
        x: 400

        Layout.fillWidth: true
    }

//    GroupBox{
//        id:groupBoxDatalog

//        anchors.left: groupBoxreg.left
//        anchors.top: groupBoxreg.bottom
//        anchors.right: groupBoxreg.right
//        title: "Datalog"


//        ColumnLayout
//        {
//            Layout.fillWidth: true
//            Button{
//            id:buttonStartDataLog
//            text: qsTr("Start DataLog")
//            Layout.fillWidth: true
//            Layout.preferredHeight: 24
//            onClicked: mcRegBank.sendUpdateDatalog(mcRegBank.getRegisterByName("ASYNC_UARTA").regId)
//            }

//            Button{
//                id:buttonStopDataLog
//                text: qsTr("Stop DataLog")
//                Layout.fillWidth: true
//                Layout.preferredHeight: 24
//                onClicked: mcRegBank.stopDatalog(mcRegBank.getRegisterByName("ASYNC_UARTA").regId)
//            }
//        }
//    }//end groupbox Datalog

}
